package main

import (
	"encoding/json"
	"fmt"
	"html/template"
	"log"
	"net/http"
	"net/url"
	"time"
)

// ImageSearchResult represents a single image result
type ImageSearchResult struct {
	Thumbnail  string
	Title      string
	Media      string
	Width      int
	Height     int
	Source     string
	ThumbProxy string
}

// QwantAPIResponse represents the JSON response structure from Qwant API
type QwantAPIResponse struct {
	Data struct {
		Result struct {
			Items []struct {
				Media     string `json:"media"`
				Thumbnail string `json:"thumbnail"`
				Title     string `json:"title"`
				Url       string `json:"url"`
				Width     int    `json:"width"`
				Height    int    `json:"height"`
			} `json:"items"`
		} `json:"result"`
	} `json:"data"`
}

var funcs = template.FuncMap{
	"sub": func(a, b int) int {
		return a - b
	},
	"add": func(a, b int) int {
		return a + b
	},
}

// FetchImageResults contacts the image search API and returns a slice of ImageSearchResult
func fetchImageResults(query string, safe, lang string, page int) ([]ImageSearchResult, error) {
	const resultsPerPage = 50
	var offset int
	if page <= 1 {
		offset = 0 // Assuming the API expects offset to start from 0 for the first page
	} else {
		offset = (page - 1) * resultsPerPage
	}

	// Ensuring safe search is enabled by default if not specified
	if safe == "" {
		safe = "1"
	}

	// Defaulting to English Canada locale if not specified
	if lang == "" {
		lang = "en_CA"
	}

	// Format &lang=lang_de is incorret, implement fix !

	apiURL := fmt.Sprintf("https://api.qwant.com/v3/search/images?t=images&q=%s&count=%d&locale=%s&offset=%d&device=desktop&tgp=2&safesearch=%s",
		url.QueryEscape(query),
		resultsPerPage,
		lang,
		offset,
		safe)

	client := &http.Client{Timeout: 10 * time.Second}

	req, err := http.NewRequest("GET", apiURL, nil)
	if err != nil {
		return nil, fmt.Errorf("creating request: %v", err)
	}

	req.Header.Set("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36")

	resp, err := client.Do(req)
	if err != nil {
		return nil, fmt.Errorf("making request: %v", err)
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		return nil, fmt.Errorf("unexpected status code: %d", resp.StatusCode)
	}

	var apiResp QwantAPIResponse
	if err := json.NewDecoder(resp.Body).Decode(&apiResp); err != nil {
		return nil, fmt.Errorf("decoding response: %v", err)
	}

	var results []ImageSearchResult
	for _, item := range apiResp.Data.Result.Items {
		results = append(results, ImageSearchResult{
			Thumbnail:  item.Thumbnail, // Thumbnail URL
			Title:      item.Title,     // Image title
			Media:      item.Media,     // Direct link to the image
			Source:     item.Url,
			ThumbProxy: "/img_proxy?url=" + url.QueryEscape(item.Media),
			Width:      item.Width,
			Height:     item.Height,
		})
	}

	return results, nil
}

// HandleImageSearch is the HTTP handler for image search requests
func handleImageSearch(w http.ResponseWriter, query, safe, lang string, page int) {
	results, err := fetchImageResults(query, safe, lang, page)
	if err != nil {
		log.Printf("Error performing image search: %v", err)
		http.Error(w, "Internal Server Error", http.StatusInternalServerError)
		return
	}

	// Parsing the template file with the custom function map
	tmpl, err := template.New("images.html").Funcs(funcs).ParseFiles("templates/images.html")
	if err != nil {
		log.Printf("Error parsing template: %v", err)
		http.Error(w, "Internal Server Error", http.StatusInternalServerError)
		return
	}

	data := struct {
		Results         []ImageSearchResult
		Query           string
		Page            int
		Fetched         string
		LanguageOptions []LanguageOption
		CurrentLang     string
		HasPrevPage     bool
		HasNextPage     bool
	}{
		Results:         results,
		Query:           query,
		Page:            page,
		Fetched:         fmt.Sprintf("%.2f seconds", time.Since(time.Now()).Seconds()),
		LanguageOptions: languageOptions,
		CurrentLang:     lang,
		HasPrevPage:     page > 1,
		HasNextPage:     len(results) >= 50,
	}

	err = tmpl.Execute(w, data)
	if err != nil {
		log.Printf("Error executing template: %v", err)
		http.Error(w, "Internal Server Error", http.StatusInternalServerError)
	}
}
